##########################################################################
# File:         utils.py                                                 #
# Purpose:      Automatically send batch of mails                        #
# Last changed: 2015/06/21                                               #
# Author:       zhuang-jia-xu                                            #
# Edited:                                                                #
# Copyleft:     (ɔ)NTUEE                                                 #
##########################################################################
import typer
from rich import print
from rich.progress import Progress, TextColumn
from rich.prompt import Prompt, Confirm
from cerberus.errors import ValidationError, ErrorList

import logging
import time


def richError(
    *objects: any, end: str = "\n", prefix: str = "Error: ", terminate: bool = True
) -> None:
    print(f"[red]{prefix}", end="")
    for o in objects:
        print(f"[red]{o}", end="")
    print("", end=end)
    if terminate:
        exit(1)


def richWarning(text: str, end: str = "\n", prefix: bool = True) -> None:
    if prefix:
        print(f"[yellow]Warning: [/yellow]", end="")
    print(f"[yellow]{text}[/yellow]", end=end)


def richSuccess(text: str, end: str = "\n") -> None:
    print(f"[green]{text}[/green]", end=end)


def parse_validation_error(errors: ErrorList) -> None:
    def _parse(error: ValidationError, indent: int = 1) -> None:
        if type(error) == ErrorList:
            for e in error:
                _parse(e, indent)
            return

        tabs = " " * 4 * indent

        if error.rule == "schema":
            richError(
                f"{tabs}{'/'.join(error.document_path)}: missing entry",
                terminate=False,
                prefix=False,
            )
        elif error.rule == "type":
            richError(
                f"{tabs}{'/'.join(error.document_path)}: should be {error.constraint}",
                terminate=False,
                prefix=False,
            )
        elif error.rule == "coerce":
            richError(
                f"{tabs}{'/'.join(error.document_path)}: ",
                *error.info,
                terminate=False,
                prefix=False,
            )
        elif error.rule == "required":
            richError(
                f"{tabs}{'/'.join(error.document_path)}: required",
                terminate=False,
                prefix=False,
            )
        else:
            richError(
                f"{tabs}{'/'.join(error.document_path)}: ",
                *error.info,
                terminate=False,
                prefix=False,
            )

        if type(error.info) == tuple and len(error.info) > 0:
            for e in error.info:
                if type(e) != str:
                    _parse(e, indent + 1)

    _parse(errors)


def complete_school_email(email_addr) -> list:
    """
    Autocomplete school email addresss
    """
    if type(email_addr) == list:
        return [complete_school_email(addr) for addr in email_addr]

    if "@" not in email_addr:
        return f"{email_addr}@ntu.edu.tw"
    else:
        return email_addr


def typerSelect(message: str, options: list) -> str:
    def process_options(n):
        n = int(n)
        if n < 0 or n >= len(options):
            raise typer.BadParameter("invalid letter index")
        return options[n]

    prompt_message = (
        "\n"
        + "\n".join(
            [
                f"{'> ' if i == 0 else '  '}[{i}] {option}"
                for i, option in enumerate(options)
            ]
        )
        + "\n\n"
        + message
    )

    result = typer.prompt(prompt_message, value_proc=process_options, default=0,)
    print()
    return result


def typerConfirm(message: str, countdown: int = 0) -> bool:
    prompt_message = f"\n{message}"
    if countdown == 0:
        result = Confirm.ask(prompt_message, default=False)
    else:
        with Progress(
            TextColumn("[bold blue]{task.description}"), transient=True
        ) as progress:
            countdown_task = progress.add_task("", total=countdown)
            for i in range(countdown + 1):
                progress.update(
                    countdown_task,
                    total=i + 1,
                    description=f' {countdown - i}{" ." * (countdown - i)}',
                )
                time.sleep(1)
        result = Confirm.ask(prompt_message, default=False)
    print()
    return result


DEBUG_LEVELS = [
    logging.NOTSET,
    logging.DEBUG,
    logging.INFO,
    logging.WARNING,
    logging.ERROR,
    logging.CRITICAL,
]


def setup_logger(filename, level):
    logging.basicConfig(
        level=DEBUG_LEVELS[level],
        filename=filename,
        format="%(asctime)s %(levelname)s: %(message)s",
        datefmt="%Y/%m/%d %I:%M:%S %p",
    )


if __name__ == "__main__":
    richSuccess("Success")
    richWarning("Warning")
    richError("Error")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ntuee_mailer']

package_data = \
{'': ['*']}

install_requires = \
['Cerberus==1.3.4',
 'PyYAML==6.0',
 'email-validator==1.2.1',
 'rich==12.5.1',
 'typer[all]>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['ntuee-mailer = ntuee_mailer.main:app']}

setup_kwargs = {
    'name': 'ntuee-mailer',
    'version': '2.0.1',
    'description': 'an auto mailer to send emails in batch for you',
    'long_description': '# `ntuee-mailer`\n\n**Installation**\n\n```bash\n$ pip install ntuee-mailer\n```\n\n**Usage**:\n\n```bash\n$ ntuee-mailer [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n- `--install-completion`: Install completion for the current shell.\n- `--show-completion`: Show completion for the current shell, to copy it or customize the installation.\n- `--help`: Show this message and exit.\n\n**Commands**:\n\n- `check`: check wether a directory is a valid letter\n- `config`: configure the auto mailer a valid config file\n- `new`: create a new letter from template\n- `send`: send emails to a list of recipients as\n\n## `ntuee-mailer check`\n\ncheck wether a directory is a valid letter\n\na letter folder should be structured as follows:\n\nletter\n\n├── attachments\n│ ├── ...\n│ └── ...\n├── config.yml\n├── content.html\n└── recipients.csv\n\n**Usage**:\n\n```console\n$ ntuee-mailer check [OPTIONS] LETTER_PATH\n```\n\n**Arguments**:\n\n- `LETTER_PATH`: Path to letter directory [required]\n\n**Options**:\n\n- `--help`: Show this message and exit.\n\n## `ntuee-mailer config`\n\nconfigure the auto mailer\n\na valid config file should have the following structure:\n\n[smtp]\nhost=smtps.ntu.edu.tw\nport=465\ntimeout=5\n\n[pop3]\nhost=msa.ntu.edu.tw\nport=995\ntimeout=5\n\n[account]\nname=John Doe\n\n**Usage**:\n\n```console\n$ ntuee-mailer config [OPTIONS]\n```\n\n**Options**:\n\n- `-f, --file TEXT`: Path to new config file whose content will be copied to config.ini\n- `-r, --reset`: Reset config.ini to default [default: False]\n- `-s, --show`: Show config.ini [default: False]\n- `--help`: Show this message and exit.\n\n## `ntuee-mailer new`\n\ncreate a new letter from template\n\n**Usage**:\n\n```console\n$ ntuee-mailer new [OPTIONS] LETTER_NAME\n```\n\n**Arguments**:\n\n- `LETTER_NAME`: Name of letter [required]\n\n**Options**:\n\n- `--help`: Show this message and exit.\n\n## `ntuee-mailer send`\n\nsend emails to a list of recipients as configured in your letter\n\n**Usage**:\n\n```console\n$ ntuee-mailer send [OPTIONS] [LETTER_PATH]\n```\n\n**Arguments**:\n\n- `[LETTER_PATH]`: Path to letter\n\n**Options**:\n\n- `-t, --test`: Test mode: send mail to yourself [default: False]\n- `-c, --config FILE`: Path to config.ini [default: /home/madmax/.config/ntuee_mailer-mailer/config.ini]\n- `-q, --quiet`: Quiet mode: less output [default: False]\n- `-d, --debug INTEGER RANGE`: Debug level [default: 0]\n- `--help`: Show this message and exit.\n',
    'author': 'zhuang-jia-xu',
    'author_email': '76544194+zhuang-jia-xu@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/NTUEE-SAAD/mailing',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
